local L = LibStub("AceLocale-3.0"):GetLocale("NeedjaTracker", true)

NTReportClass = {}

function NTReportClass:Create(...)
    local ret = CloneTable(NTReportClass)
    if ret.Initialize then
        ret:Initialize(...)
    end
    return ret
end

function NTReportClass:Initialize(parent, watch)
    self.parent = parent
    self.watch = watch

    LibStub("AceEvent-3.0"):Embed(self)

    self:RegisterMessage("NT_ROLL_PASSED", "EventPassed")
    self:RegisterMessage("NT_ROLL_CHOSE", "EventChose")
    self:RegisterMessage("NT_ROLL_WON_NO_SPAM", "EventWonNoSpam")
end

function NTReportClass:ReportStatistics(channel)
    local stat = self.watch:GetStatistics()
    if # stat == 0 then
        PrintToChat(L.msg_stat_head_empty, channel)
        return
    end

    PrintToChat(L.msg_stat_head, channel)

    table.sort(stat, function(a, b) return a.need > b.need end)
    local top = stat[1].need
    table.sort(stat, function(a, b) return a.win > b.win end)

    local n, data
    for n, data in pairs(stat) do
        if top > 0 and data.need == top then
            PrintToChat((L.msg_stat_top):format(
                n, data.player, data.win, data.need, data.greed,
                data.disenchant, data.pass
            ), channel)
        elseif data.need > 0 then
            PrintToChat((L.msg_stat_needja):format(
                n, data.player, data.win, data.need, data.greed,
                data.disenchant, data.pass
            ), channel)
        else
            PrintToChat((L.msg_stat_line):format(
                n, data.player, data.win, data.need, data.greed,
                data.disenchant, data.pass
            ), channel)
        end
    end
end

function NTReportClass:ReportHistory(channel, full)
    local hist = self.watch:GetHistory()
    if # hist == 0 then
        PrintToChat(L.msg_hist_head_empty, channel)
        return
    end

    PrintToChat(L.msg_hist_head, channel)

    local n, data
    for n, data in pairs(hist) do
        if data.choice == "pass" then
            PrintToChat((L.msg_hist_pass):format(n, data.item), channel)

        elseif data.choice == "need" then
            PrintToChat((L.msg_hist_need):format(
                n, data.item, data.player, L["roll_" .. data.choice], data.roll
            ), channel)
        else
            PrintToChat((L.msg_hist_win):format(
                n, data.item, data.player, L["roll_" .. data.choice], data.roll
            ), channel)
        end
        if not full and n >= 10 then break end
    end
end

function NTReportClass:EventPassed(event, item)
    PrintToChat((L.msg_pass_party):format(item))
end

function NTReportClass:EventChose(event, item, player, choice)
    if choice == "need" and player ~= UnitName("player") then
        PrintToChat((L.msg_roll_need):format(item, player, L.roll_need))
    end
end

function NTReportClass:EventWonNoSpam(event, item, player, choice, roll)
    if player == UnitName("player") then
        PrintToChat((L.msg_win_you):format(
            item, L["roll_" .. choice], roll
        ))
    elseif choice == "need" then
        PrintToChat((L.msg_win_party_need):format(
            item, player, L["roll_" .. choice], roll
        ))
    else
        PrintToChat((L.msg_win_party):format(
            item, player, L["roll_" .. choice], roll
        ))
    end
end
