local L = LibStub("AceLocale-3.0"):GetLocale("NeedjaTracker", true)

NTWatchClass = {}

function NTWatchClass:Create(...)
    local ret = CloneTable(NTWatchClass)
    if ret.Initialize then
        ret:Initialize(...)
    end
    return ret
end

function NTWatchClass:Initialize(parent, db)
    self.parent = parent
    self.db = db

    LibStub("AceEvent-3.0"):Embed(self)

    self:RegisterMessage("NT_ROLL_PASSED", "EventPassed")
    self:RegisterMessage("NT_ROLL_CHOSE", "EventChose")
    self:RegisterMessage("NT_ROLL_ROLLED", "EventRolled")
    self:RegisterMessage("NT_ROLL_WON", "EventWon")
    self:RegisterMessage("NT_ROLL_WON_NO_SPAM", "EventWonNoSpam")

    self:RegisterMessage("NT_JOIN_PARTY", "EventJoinParty")
    self:RegisterMessage("NT_JOIN_RAID", "EventJoinRaid")
    self:RegisterMessage("NT_LEAVE_PARTY", "EventLeaveParty")
    self:RegisterMessage("NT_LEAVE_RAID", "EventLeaveRaid")
end

function NTWatchClass:GetStatistics()
    return GetSubtable(self.db.char, "RollStatistics")
end

function NTWatchClass:GetHistory()
    return GetSubtable(self.db.char, "RollHistory")
end

function NTWatchClass:ClearStatistics()
    self.db.char.RollStatistics = {}
    self.db.char.RollHistory = {}

    self:SendMessage("NT_UPDATE_STATISTICS")
    self:SendMessage("NT_UPDATE_HISTORY")
end

function NTWatchClass:AddStatistics(player, choice)
    local stat = GetSubtable(self.db.char, "RollStatistics")
    local n, data, found
    for n, data in pairs(stat) do
        if data.player == player then
            found = true
            data[choice] = data[choice] + 1
            break
        end
    end
    if not found then
        table.insert(stat, { player = player, win = 0, need = 0, greed = 0, disenchant = 0, pass = 0, [choice] = 1 })
    end
    self:SendMessage("NT_UPDATE_STATISTICS")
end

function NTWatchClass:AddHistory(item, choice, player, roll)
    table.insert(GetSubtable(self.db.char, "RollHistory"), 1,
        { item = item, choice = choice, player = player, roll = roll })
    self:SendMessage("NT_UPDATE_HISTORY")
end

function NTWatchClass:EventPassed(event, item)
    GetSubtable(self.db.char, "RollResults")[item] = nil
    self:AddHistory(item, "pass")
end

function NTWatchClass:EventChose(event, item, player, choice)
    local res = GetSubtable(self.db.char, "RollResults", item, player)
    res.choice = choice
    self:AddStatistics(player, choice)
end

function NTWatchClass:EventRolled(event, item, player, choice, roll)
    local res = GetSubtable(self.db.char, "RollResults", item, player)
    res.choice, res.roll = choice, roll
end

function NTWatchClass:EventWon(event, item, player)
    local res = GetSubtable(self.db.char, "RollResults", item, player)
    self:SendMessage("NT_ROLL_WON_NO_SPAM", item, player,
        res.choice or "unknown", res.roll or "?")
end

function NTWatchClass:EventWonNoSpam(event, item, player, choice, roll)
    GetSubtable(self.db.char, "RollResults")[item] = nil
    self:AddStatistics(player, "win")
    self:AddHistory(item, choice, player, roll)
end

function NTWatchClass:EventJoinParty()
    self.db.char.RollResults = {}
end

function NTWatchClass:EventJoinRaid()
    self.db.char.RollResults = {}
end

function NTWatchClass:EventLeaveParty()
    self.db.char.RollResults = {}
end

function NTWatchClass:EventLeaveRaid()
    self.db.char.RollResults = {}
end
