local L = LibStub("AceLocale-3.0"):GetLocale("NeedjaTracker", true)

NeedjaTracker = LibStub("AceAddon-3.0"):NewAddon("NeedjaTracker", "AceConsole-3.0", "AceEvent-3.0")

function NeedjaTracker:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("NeedjaTrackerDB", self.defaults, true)

    LibStub("AceConfig-3.0"):RegisterOptionsTable("Needja Tracker", self.options)
    LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Needja Tracker", "Needja Tracker")

    ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", function(this, ...) return self:FilterChatMessage(...) end)

    self:RegisterEvent("CHAT_MSG_LOOT", "ProcessChatMessage")

    self:RegisterEvent("PLAYER_ENTERING_WORLD", "ProcessInstanceEvent")
    self:RegisterEvent("RAID_ROSTER_UPDATE", "ProcessPartyEvent")
    self:RegisterEvent("PARTY_MEMBERS_CHANGED", "ProcessPartyEvent")

    self.watch = NTWatchClass:Create(self, self.db)
    self.report = NTReportClass:Create(self, self.watch)
    self.gui = NTGuiMainClass:Create(self, self.watch)
    self.reset = NTGuiResetClass:Create(self, self.watch)
end

function NeedjaTracker:FilterChatMessage(event, msg)
    return ParseMsgMatchTable(LOOT_ROLL_MSG_MATCH_TABLE, msg)
end

function NeedjaTracker:ProcessChatMessage(event, msg)
    local handler, args = ParseMsgMatchTable(LOOT_ROLL_MSG_MATCH_TABLE, msg)

    if handler == "LOOT_ROLL_CHOSE" then
        self:SendMessage("NT_ROLL_CHOSE", args.item, args.player, args.choice)

    elseif handler == "LOOT_ROLL_ROLLED" then
        self:SendMessage("NT_ROLL_ROLLED", args.item, args.player, args.choice, args.roll)

    elseif handler == "LOOT_ROLL_WON" then
        self:SendMessage("NT_ROLL_WON", args.item, args.player)

    elseif handler == "LOOT_ROLL_WON_NO_SPAM" then
        self:SendMessage("NT_ROLL_WON_NO_SPAM", args.item, args.player, args.choice, args.roll)

    elseif handler == "LOOT_ROLL_ALL_PASSED" then
        self:SendMessage("NT_ROLL_PASSED", args.item)
    end
end

function NeedjaTracker:ProcessInstanceEvent()
    local db = self.db.char
    local instance = NT_IsInInstance()
    if instance then
        if db.InstanceStatus ~= instance then
            self:SendMessage("NT_ENTER_INSTANCE")
            db.InstanceStatus = instance
        end
    end
end

function NeedjaTracker:ProcessPartyEvent()
    local db = self.db.char
    local status = db.PartyStatus or 0
    if NT_IsInRaid() then
        if status < 3 then
            self:SendMessage("NT_JOIN_RAID")
            db.PartyStatus = 3
        end
    elseif NT_IsInLFGParty() then
        if status < 2 then
            self:SendMessage("NT_JOIN_PARTY")
            db.PartyStatus = 2
        end
    elseif NT_IsInParty() then
        if status < 1 then
            self:SendMessage("NT_JOIN_PARTY")
            db.PartyStatus = 1
        end
    else
        if status > 2 then
            self:SendMessage("NT_LEAVE_RAID")
            db.PartyStatus = nil
        elseif status > 0 then
            self:SendMessage("NT_LEAVE_PARTY")
            db.PartyStatus = nil
        end
    end
end
